#!/bin/perl
#
# make_depend.pl
#
# Produce a file to include from the makefile containing updated dependencies
# for all the files. This looks for all #import or #include statements with
# double quotes around the name of the header file, and produces a line like
# this:
#
# MyObject.o: MyObject.h MyObject.m <Everything in quotes from MyObject.h> \
#                           <Everything in quotes from MyObject.m>
#
# The command is called with a list of object files. This will be $(OBJECTS) 
# in the Makefile

$makedependfile = "Makefile.depend";

open(MAKEDEPEND, ">$makedependfile")
  || die "Cannot create $makedependfile: $!\n";
foreach $obj (@ARGV) {
  $interface = $obj;
  $interface =~ s/o$/h/;
  $implement = $obj;
  $implement =~ s/o$/m/;
  if(!open(INTERFACE, "<$interface")) {
    print STDERR
      "Cannot read interface file for $obj: $interface (Error: $!)\n";
  }
  else {
    @interimports = grep(/^\#\s*(import|include)\s+\"/, <INTERFACE>);
    close(INTERFACE);
  }
  if(!open(IMPLEMENT, "<$implement")) {
    print STDERR
      "Cannot read implementation file for $obj: $implement (Error: $!)\n";
    next;
  }
  else {
    @impleimports = grep(/^\#\s*(import|include)\s+\"/, <IMPLEMENT>);
    close(IMPLEMENT);
  }
  print MAKEDEPEND "$obj: $implement";
  foreach $header (@impleimports, @interimports) {
    if($header =~ /\"(.*\.h)\"/) {
      $headerfile = $1;
    }
    else {
      next;
    }
    print MAKEDEPEND " $headerfile";
  }
  print MAKEDEPEND "\n";
}
close(MAKEDEPEND);
exit(0);

